local AddonName  = ...
local actionAura = LibStub("AceAddon-3.0"):NewAddon(AddonName)

actionAura.isUnitPlayer = {
	player = true,
	vehicle = true,
	pet = true,
}

actionAura.units = {
		"mouseover",
		"target",
		"focus" ,
		"player",
 }

actionAura.GetUnit = {
	HARMFUL = function()
		for i, unitTarget in pairs(actionAura.units) do
			if (UnitExists(unitTarget) == true)
			and(UnitIsDeadOrGhost(unitTarget) ~= true)
			and(UnitCanAttack(unitTarget, "player") == true) then
				return unitTarget, i
			end
		end
		--if no unit is found, return a filler unit to prevent errors
		return "target"
	end,
	HELPFUL = function()
		for i, unitTarget in pairs(actionAura.units) do
			if (UnitExists(unitTarget) == true)
			and(UnitIsDeadOrGhost(unitTarget) ~= true)
			and(UnitCanAttack(unitTarget, "player") ~= true) then
				return unitTarget, i
			end
		end
		--if no unit is found, return a filler unit to prevent errors
		return "player"
	end,
}

function actionAura:GetActionIndex(actionButton)
	local name = actionButton.name or actionButton:GetName()
	if name and string.find(name, 'BT4Button', 1) then
		local actionType, actionID = actionButton:GetAction()
		if actionType == 'action' then
			return actionID
		end
	else
		return actionButton.action
	end
end

function actionAura:GetAuraTarget(actionType, actionID, filter, auraName)
	if actionType == 'macro' then
		local macroName = GetMacroInfo(actionID)
		if macroName ~= nil then
			-- check if we have 'Focus' in the macro name, if so, target is 'focus' for this button
				--may now be any of four units in the macro title. (mouseover, target, focus, player) ~Goranaws
			for i, unitTarget in pairs(self.units) do
				local startPos, endPos = string.find(string.lower(macroName), unitTarget)
				if startPos ~= nil then
					return unitTarget
				end
			end
		end
	end
	return self.GetUnit[filter]()
end

function actionAura:GetAuraDetails(spellName, actionType, actionID)
	if spellName and actionAura_SV.translations[spellName] and (#actionAura_SV.translations[spellName] > 0) then
		for i, overRideSpellName in pairs(actionAura_SV.translations[spellName]) do
			if overRideSpellName then
				local filter = IsHarmfulSpell(spellName) and "HARMFUL" or "HELPFUL"
				local unit = self:GetAuraTarget(actionType, actionID, filter, overRideSpellName)
				local name, _, _, _, duration, expireTime, source, _, _, _, _, _, castByPlayer = AuraUtil.FindAuraByName(overRideSpellName, unit, filter)
				if name then
					return name, duration, expireTime, (castByPlayer == true) and "player" or source
				end
			end
		end
	end
	if spellName then
		local filter = IsHarmfulSpell(spellName) and "HARMFUL" or "HELPFUL"
		local unit = self:GetAuraTarget(actionType, actionID, filter, spellName)
		local name, _, _, _, duration, expireTime, source, _, _, _, _, _, castByPlayer = AuraUtil.FindAuraByName(spellName, unit, filter)
		return name, duration, expireTime, (castByPlayer == true) and "player" or source
	end
end

function actionAura:GetAuraName(actionButton)
	local actionIndex = self:GetActionIndex(actionButton)
	if actionIndex then
		local actionType, actionID, _, globalID = GetActionInfo(actionIndex) --Returns information about a specific action.
		local spellName
		if actionType == 'spell' then
			if actionID and actionID > 0 then
				spellName = GetSpellInfo(actionID)
			elseif globalID then
				spellName = GetSpellInfo(globalID)
			end
		elseif actionType == 'item' then
			spellName = GetItemSpell(actionID)
		elseif actionType == 'macro' then
			actionID = GetMacroSpell(actionID)
			if actionID then
				spellName = GetSpellInfo(actionID)
			end
		end
		local auraName, auraDuration, auraExpireTime, source = self:GetAuraDetails(spellName, actionType, actionID)
		if auraName and self.isUnitPlayer[source] then
			return auraName, auraExpireTime, auraDuration, GetSpellCooldown(spellName)
		end
	end
end

actionAura.buttonRegistry = {}
function actionAura:SetAuraCooldown(actionButton, expireTime, duration)
	if actionButton.auraCooldown == nil then
		if not tContains(self.buttonRegistry, actionButton) then
			tinsert(self.buttonRegistry, actionButton)
			local name = actionButton.name or actionButton:GetName()
			actionButton.spellCooldown = name and _G[name .. 'Cooldown'] or actionButton.cooldown -- catches more things.
			actionButton.auraCooldown = CreateFrame('Cooldown', nil, actionButton, 'CooldownFrameTemplate')
			actionButton.auraCooldown:SetAllPoints(actionButton)
			actionButton.auraCooldown:SetScript("OnShow", function()
				actionButton.spellCooldown:Hide()
			end)
		end
	end
	actionButton.auraCooldown:SetCooldown(expireTime - duration, duration)
end

function actionAura:UpdateActionAura(actionButton)
	local auraName, auraExpireTime, auraDuration, spellStart, spellDuration = self:GetAuraName(actionButton)
	if auraName and ((auraExpireTime and spellStart) and (auraExpireTime > spellStart + spellDuration)) then
		self:SetAuraCooldown(actionButton, auraExpireTime, auraDuration)
	elseif actionButton.auraCooldown and actionButton.auraCooldown:IsShown() then
		actionButton.auraCooldown:SetCooldown(0, 0)
		actionButton.auraCooldown:Hide()
	end
end

function actionAura:OnInitialize()
	if not self.eventHandler then
		self:RegisterVariables(reset)

		hooksecurefunc("ActionButton_UpdateCooldown", function(actionButton)
			self:UpdateActionAura(actionButton)
		end)

		--the following is for fringe cases
		self.eventHandler = CreateFrame("Frame")
		self.eventHandler:RegisterEvent("PET_BAR_UPDATE_USABLE")
		self.eventHandler:RegisterEvent("PET_UI_UPDATE")
		self.eventHandler:RegisterEvent("UNIT_AURA")
		self.eventHandler:RegisterEvent("UNIT_COMBAT")
		self.eventHandler:RegisterEvent("UNIT_PET")
		self.eventHandler:RegisterEvent("UNIT_SPELLCAST_SENT")
		self.eventHandler:RegisterEvent("UNIT_ENTERED_VEHICLE")
		self.eventHandler:RegisterEvent("UNIT_HEALTH")
		self.eventHandler:RegisterEvent("UNIT_TARGET")
		self.eventHandler:RegisterEvent("PLAYER_FOCUS_CHANGED")

		self.eventHandler:SetScript("OnEvent", function()
			for _, button in pairs(self.buttonRegistry) do
				self:UpdateActionAura(button)
			end
		end)
	end
end

do --settings access
	actionAura.defaults = {
		translations = {
			["Entangling Roots"] = {"Mass Entanglement"},
			["testering"] = {
				"1",
				"2",
				"3",
				"4",
				"5",
				"6",
				"7",
				"8",
				"9",
				"10",
				"11",
				"12",
				"13",
				"14",
				"15",
				"16",
				"17",
				"18",
				"19",
				"20"
			}

		},
	}

	function actionAura:RegisterVariables(reset)
		if reset == true then
			actionAura_SV = nil
		end
		actionAura_SV = actionAura_SV or {}
		if self.defaults then
			for key, value in pairs(self.defaults) do
				actionAura_SV[key] = actionAura_SV[key] or value
			end
		end
	end

	function actionAura:Get(key, ...)
		return actionAura_SV[key] or self.defaults and self.defaults[key]
	end

	function actionAura:Set(key, value)
		actionAura_SV[key] = value
	end

	function actionAura:Reset()
		self:RegisterVariables(true)
	end
end

function actionAura:AddTranslation(spellName, translation)
	actionAura_SV.translations[spellName] = actionAura_SV.translations[spellName] or {}
	if translation then
		if not tContains(actionAura_SV.translations[spellName], translation) then
			tinsert(actionAura_SV.translations[spellName], translation)
		end
	end
end

local function CreateButton(title, parent)
	local button = CreateFrame("Frame", AddonName..title.."Button", parent, "TooltipBorderedFrameTemplate")
	button:SetHeight(25)

	button.edit = CreateFrame("Button", AddonName..title.."ButtonEdit", button)
		button.edit:SetSize(15, 15)
		button.edit:SetPoint("Right", -5, 0)
		button.edit:SetPushedAtlas("NPE_ArrowDown")
		button.edit:SetNormalAtlas("NPE_ArrowDownGlow")
		button.edit:SetHighlightAtlas("bags-glow-artifact")



	button.text = CreateFrame("EditBox", AddonName..title.."ButtonText", button, "InputBoxScriptTemplate")
		button.text:SetHeight(15)
		button.text:SetAutoFocus(false)
		button.text:SetTextColor(1,1,1,1)
		button.text:SetHighlightColor(.5,1,.5)
		button.text:SetFontObject("GameFontNormal")
		button.text:SetPoint("Left", 5, 0)
		button.text:SetPoint("Right", button.edit, "Left", 0, 0)

		button.text.high = button.text:CreateTexture(nil, 'HIGHLIGHT', 2)

		button.text.high:SetAllPoints(button.text)

		button.text.high:SetAtlas("soulbinds_collection_entry_highlight")

		button.text:SetHitRectInsets(-5, -15, -5, -5)


	return button
end


function actionAura:ShowTranslationPanel()
	if not self.TransPanel then
		self.TransPanel = CreateFrame("Frame", AddonName.."TransPanel", UIParent, "TooltipBorderedFrameTemplate")
		self.TransPanel:SetSize(300, 300)
		self.TransPanel:SetPoint("Center")

		local textTitle = self.TransPanel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		textTitle:SetPoint("TopLeft", 5, -5)
		textTitle:SetText("Action Aura: Translations")

		local closeButton = CreateFrame("Button", nil, self.TransPanel, "UIPanelCloseButton")
		closeButton:SetPoint("TopRight", 2, 2)
		closeButton:SetScale(.8)

		local topX = -25

		local t = self.TransPanel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		t:SetPoint("TopLeft", 5, topX - 5)
		t:SetText("Original:")

		local u = self.TransPanel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		u:SetPoint("TopLeft", 5, -55)
		u:SetText("Translation:")

		self.TransPanel:SetMovable(true)
		self.TransPanel:SetScript("OnMouseDown", self.TransPanel.StartMoving)
		self.TransPanel:SetScript("OnMouseUp", self.TransPanel.StopMovingOrSizing)

		self.Original = CreateButton("TransPanelOriginalSpell", self.TransPanel)
		self.Original:SetHeight(25)
		self.Original:SetPoint("Left", t, "Right", 3, 0)
		self.Original:SetPoint("TopRight", -20, topX)


		self.delete = CreateFrame("Button", AddonName.."DeleteEntry", self.TransPanel)
		self.delete:SetSize(15, 15)
		self.delete:SetPoint("Left", self.Original,"Right")
		self.delete:SetPushedAtlas("NPE_ArrowDown")
		self.delete:SetNormalAtlas("NPE_ArrowDownGlow")
		self.delete:SetHighlightAtlas("bags-glow-artifact")

		self.delete:SetNormalAtlas("BackupPet-DeadFrame")
		self.delete:SetPushedAtlas("BattleBar-SwapPetFrame-DeadIcon")
		self.delete:SetDisabledAtlas("Objective-Fail")

		self.delete:GetDisabledTexture():SetDesaturated(true)

		self.delete:Disable()

		self.delete:SetScript("OnEnter",  function()
			GameTooltip:SetOwner(self.delete, "ANCHOR_RIGHT")
			GameTooltip:SetText("Click to delete all entries for this spell.")

			GameTooltip:Show()
		end)


		self.delete:SetScript("OnLeave",  function()
			GameTooltip:Hide()
		end)

		self.Translation = CreateButton("TransPanelOriginalSpell", self.TransPanel)
		self.Translation:SetHeight(25)
		self.Translation:SetPoint("TopLeft", self.Original, "BottomLeft", 20, 0)
		self.Translation:SetPoint("TopRight", -5, topX - 25)
		--self.Translation:SetPoint("TopRight", self.Original, "BottomRight", 0, 0)

		

		self.TransPanel.Original = self.Original
		self.TransPanel.Translation = self.Translation

		function self.TransPanel:Insert(text)
			local orig = self.Original.text:GetText() 
			local tran = self.Translation.text:GetText()

			if not orig or orig == "" then
				self.Original.text:SetText(text)
			else
				self.Translation.text:SetText(text)
			end
		end
		
		function self.TransPanel:SetFocus(text)

		end



		self.Translation.edit:SetNormalAtlas("tradeskills-icon-add")
		self.Translation.edit:SetNormalAtlas("bags-icon-addslots")

		local line = self.TransPanel:CreateLine(nil, 'ARTWORK', 1)
		line:SetThickness(2)
		line:SetStartPoint("TopLeft", 8, -53 + topX)
		line:SetEndPoint("TopRight", -8, -53 + topX)

		line:SetColorTexture(.8,.8,.8,.4)

		local s = self.TransPanel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		s:SetPoint("Top", line, "Bottom", 0, -5)
		s:SetText("Translations")

		local buttons = {}
		local displayOffset = 0

		local slide = CreateFrame("Slider", self.TransPanel:GetName().."Scroll", self.TransPanel, "HorizontalSliderTemplate")

		slide:Hide()
		slide:SetWidth(18)
		slide:ClearAllPoints()
		slide:SetPoint("Top", 0, -65 + topX)
		slide:SetPoint("BottomRight", -8, 3)
		slide:SetOrientation("VERTICAL")

		slide:SetScript("OnMouseWheel", function(_, delta)
			slide:SetValue(floor(slide:GetValue()-delta))
		end)

		self.TransPanel:SetScript("OnMouseWheel", function(_, delta)
			slide:SetValue(floor(slide:GetValue()-delta))
		end)

		local numButtons = 8

		local wi, he = self.TransPanel:GetSize()

		local h = (he - (75 - topX))/numButtons


		local w = (wi - 33)

		self.Translation.text:SetScript("OnTextChanged", function()
			local text = self.Translation.text:GetText()
			text = (strfind(text, "spell:", 1, true)) and GetSpellInfo(tonumber(text:match("spell:(%d+)")))
			or (text:match(":(%d+)")) and "" ----it is not a spell link, don't allow it
			or text or ""
			
			local _ = text ~= self.Translation.text:GetText() and self.Translation.text:SetText(text)			
		end)

		local CurrentTranslationList
		local lastSpell
		local function RegisterNew(spellName, translation)
			if spellName and spellName ~= "" then
				actionAura:AddTranslation(spellName or lastSpell, translation)
				lastSpell = spellName or lastSpell
				local translations = actionAura_SV.translations[spellName]
				CurrentTranslationList = translations
			end
			if CurrentTranslationList then
				local oWidth = he - 33
				if #CurrentTranslationList > numButtons then
					slide:Enable()
					local v = slide:GetValue()
					slide:SetMinMaxValues(0, #CurrentTranslationList - numButtons)

					if (not v) or (v == 0) or v > #CurrentTranslationList - numButtons then
						slide:SetValue(0)
					end
					slide:Show()
					oWidth = he - 33
				else
					slide:Hide()
					oWidth = he - 10
				end

				for i = 1, numButtons do
					local b = buttons[i]
					b:SetWidth(oWidth)
					b.text:ClearFocus()
					if CurrentTranslationList[i + displayOffset] then
						b:Show()
						b.text:SetText(CurrentTranslationList[i + displayOffset])
						b.text:SetCursorPosition(0)
					else
						b:Hide()
						b.text:SetText("")
					end
				end

			else
				for i = 1, numButtons do
					local b = buttons[i]
					b:Hide()
				end
				self.delete:Disable()

			end
		end
		slide.stepSize = 1
		self.TransPanel.stepSize = 1
		self.TransPanel.min = 0
		self.TransPanel.stepSize = 1


		self.Original.text:SetScript("OnTextChanged", function()
			local text = self.Original.text:GetText()

			text = (strfind(text, "spell:", 1, true)) and GetSpellInfo(tonumber(text:match("spell:(%d+)")))
			or (text:match(":(%d+)")) and "" ----it is not a spell link, don't allow it
			or text or ""

			if text and text ~= "" then
				local _ = text ~= self.Original.text:GetText() and (self.Original.text:SetText(text) == nil and self.Translation.text:SetFocus())
				RegisterNew(text)
				if actionAura_SV.translations[text] then
					self.delete:Enable()
				else
					self.delete:Disable()
				end
			else

				for i = 1, numButtons do
					local b = buttons[i]
					b:Hide()
				end

				slide:Hide()

				self.delete:Disable()
			end
		end)

		slide:SetScript("OnValueChanged", function()
			displayOffset = floor(slide:GetValue())
			RegisterNew()
		end)

		self.delete:SetScript("OnClick", function()

			local text = self.Original.text:GetText()

			if text:match(":(%d+)") then --it is a link of some sort
				if strfind(text, "spell:", 1, true) then
					-- it is a spell link
					text = GetSpellInfo(tonumber(text:match("spell:(%d+)")))
				else
					--it is not a spell link, don't allow it
					text = nil
				end
			end


			if text and text ~= "" then
				if actionAura_SV.translations[text] then
					actionAura_SV.translations[text] = nil
					RegisterNew("")
					self.Original.text:SetText("")
				end
			else
				RegisterNew("")
			end
		end)


		for i = 1, numButtons do
			buttons[i] = CreateButton("Button"..i, self.TransPanel) --CreateFrame("Frame", nil, c, "TooltipBorderedFrameTemplate")
			buttons[i]:SetSize(w, h)
			buttons[i]:SetPoint("TopLeft", 5, -(70 - topX) - ((h * (i - 1))))

			buttons[i].edit:SetPushedAtlas("BackupPet-DeadFrame")
			buttons[i].edit:SetNormalAtlas("BattleBar-SwapPetFrame-DeadIcon")

			buttons[i].edit:SetScript("OnClick", function()
				tremove(CurrentTranslationList, i + displayOffset)
				RegisterNew()
			end)

			buttons[i].text:SetScript("OnEnterPressed", function()
				CurrentTranslationList[i + displayOffset] = buttons[i].text:GetText()
				buttons[i].text:ClearFocus()
				buttons[i].text:SetCursorPosition(0)
			end)

			buttons[i].text:SetScript("OnEditFocusLost", function()
				buttons[i].text:SetText(CurrentTranslationList[i + displayOffset] or "")
			end)

			buttons[i]:Hide()
		end

		self.Original.text:SetScript("OnEditFocusGained", function()
			ACTIVE_CHAT_EDIT_BOX = self.Original.text
		end)

		self.Original.text:SetScript("OnEditFocusLost", function()
			ACTIVE_CHAT_EDIT_BOX = nil
			if self.TransPanel:IsVisible() then
				ACTIVE_CHAT_EDIT_BOX = self.TransPanel
			end
		end)

		self.Original.text:SetScript("OnEnterPressed", function()
			local text = self.Original.text:GetText()

			if text:match(":(%d+)") then --it is a link of some sort
				if strfind(text, "spell:", 1, true) then
					-- it is a spell link
					text = GetSpellInfo(tonumber(text:match("spell:(%d+)")))
				else
					--it is not a spell link, don't allow it
					text = nil
				end
			end


			if text then
				RegisterNew(text)
				self.Original.text:SetText(text)
				self.Translation.text:SetFocus()
			else
				self.Original.text:SetText("")
				self.Original.text:ClearFocus()
			end
		end)

		self.Original.edit:SetScript("OnMouseDown", function()
			if not self.dropDown then
				self.dropDown = CreateFrame("Frame", AddonName.."DropDown", self.Original, "UIDropDownMenuTemplate")
				UIDropDownMenu_Initialize( self.dropDown, function(_, level, ...)
						UIDropDownMenu_AddButton({
							text = "Library",
							notCheckable = true,
							hasArrow = nil,
							isTitle = true,
							func = function()

							end,
						})
					for spellName, translations in pairs(actionAura_SV.translations) do
						UIDropDownMenu_AddButton({
							text = spellName,
							notCheckable = true,
							hasArrow = nil,
							func = function()
								self.Original.text:SetText(spellName)
								RegisterNew(spellName, nil)
							end,
						})
					end

				end, "MENU")
			end
			ToggleDropDownMenu(1, 1, self.dropDown, self.Original:GetName(), 0, 0)
		end)

		self.TransPanel:SetScript("OnShow", function()
			ACTIVE_CHAT_EDIT_BOX = self.TransPanel
		end)

		self.TransPanel:SetScript("OnHide", function()
			ACTIVE_CHAT_EDIT_BOX = nil
		end)

		self.Translation.text:SetScript("OnEditFocusGained", function()
			ACTIVE_CHAT_EDIT_BOX = self.Translation.text
		end)

		self.Translation.text:SetScript("OnEditFocusLost", function()
			ACTIVE_CHAT_EDIT_BOX = nil
			if self.TransPanel:IsVisible() then
				ACTIVE_CHAT_EDIT_BOX = self.TransPanel
			end
		end)

		self.Translation.text:SetScript("OnEnterPressed", function()
			if self.Original.text:GetText() ~= "" and self.Translation.text:GetText() ~= "" then
				local text = self.Translation.text:GetText()
				text = (strfind(text, "spell:", 1, true)) and GetSpellInfo(tonumber(text:match("spell:(%d+)")))
				or (text:match(":(%d+)")) and nil
				or text

				RegisterNew(self.Original.text:GetText(), text)
				self.Translation.text:SetText("")
			end
			self.Translation.text:ClearFocus()
		end)

		self.Translation.edit:SetScript("OnClick", function()
			if self.Original.text:GetText() ~= "" and self.Translation.text:GetText() ~= "" then
				RegisterNew(self.Original.text:GetText(), self.Translation.text:GetText())
				self.Translation.text:SetText("")
			end
			self.Translation.text:ClearFocus()
		end)

	end

	self.TransPanel:Show()
end

function actionAura:RunCommand(Cmd, ...)
	local cmdStrings = string.lower(Cmd)
	if string.find(cmdStrings, "reset", 1) then
		self:Reset()
	else
		actionAura:ShowTranslationPanel()
	end
end

SlashCmdList[string.upper(AddonName)] = function(Cmd, ...)
	actionAura:RunCommand(Cmd, ...)
end

_G["SLASH_".. string.upper(AddonName) .."1"] = "/aaura"
_G["SLASH_".. string.upper(AddonName) .."2"] = "/"..string.lower(AddonName)
